//
//  AbbDatePickView.swift
//  AbbDatePickDemo
//
//  Created by abb on 16/8/3.
//  Copyright © 2016年 abb. All rights reserved.
//

import UIKit
/**
 边角类型
 
 - Normal: 默认
 - Circle: 切角
 */
enum CornorType:Int {
    case Normal
    case Circle
}
class AbbDatePickView: UIView {
    /// 日历
    var calendar:NSCalendar?
    /// 最小日期
    var minDate:NSDate?
    /// 最大日期
    var maxDate:NSDate?
    /// 默认最早的日期
    var earliestPresentedDate:NSDate {
        return self.showOnlyValidDates ? self.minDate! : NSDate(timeIntervalSince1970: 0)
    }
    /// 显示日期行数
    var nDays:Int?
    /// 是否只显示有效日期
    var showOnlyValidDates:Bool = false
    /// 保存最终返回的日期
    var date:NSDate?
    /// 字体配色
    var color:UIColor = UIColor.whiteColor()
    var bgColor:UIColor = UIColor.blackColor()
    /// 回调闭包
    var didFinishSelect:((date:NSDate)->())?
    // MARK: - 初始化方法
    init(cornorType:CornorType,minDate: NSDate,maxDate: NSDate,showOnlyValidDates: Bool) {
        super.init(frame: UIScreen.mainScreen().bounds)
        self.minDate = minDate
        self.maxDate = maxDate
        self.showOnlyValidDates = showOnlyValidDates
        if cornorType == CornorType.Circle {
            setupCornor(bgView.frame)
        }
        setupUI()
        initDate()
        showDateOnPicker(self.date!)
    }
    init(cornorType:CornorType) {
        super.init(frame: UIScreen.mainScreen().bounds)
        if cornorType == CornorType.Circle {
            setupCornor(bgView.frame)
        }
        setupUI()
        initDate()
        showDateOnPicker(self.date!)
    }
    /**
     加载UI
     */
    private func setupUI() {
        // 添加子控件
        addSubview(bgView)
        bgView.addSubview(cancelBtn)
        bgView.addSubview(titleLabel)
        bgView.addSubview(donelBtn)
        bgView.addSubview(splitLine)
        bgView.addSubview(pickView)
        
        // 布局子控件
        cancelBtn.translatesAutoresizingMaskIntoConstraints = false
        donelBtn.translatesAutoresizingMaskIntoConstraints = false
        titleLabel.translatesAutoresizingMaskIntoConstraints = false
        splitLine.translatesAutoresizingMaskIntoConstraints = false
        pickView.translatesAutoresizingMaskIntoConstraints = false
        
        var cons = [NSLayoutConstraint]()
        let viewDict = ["cancelBtn":cancelBtn,"donelBtn":donelBtn,"titleLabel":titleLabel,"splitLine":splitLine,"pickView":pickView]
        cons += NSLayoutConstraint.constraintsWithVisualFormat("H:|-20-[cancelBtn]-20-[titleLabel(cancelBtn)]-20-[donelBtn(cancelBtn)]-20-|", options: NSLayoutFormatOptions(rawValue: 0), metrics: nil, views: viewDict)
        cons += NSLayoutConstraint.constraintsWithVisualFormat("V:|-10-[cancelBtn(30)]-5-[splitLine(0.5)]-5-[pickView]-0-|", options: NSLayoutFormatOptions(rawValue: 0), metrics: nil, views: viewDict)
        cons += NSLayoutConstraint.constraintsWithVisualFormat("V:|-10-[titleLabel(30)]-5-[splitLine(0.5)]-5-[pickView]-0-|", options: NSLayoutFormatOptions(rawValue: 0), metrics: nil, views: viewDict)
        cons += NSLayoutConstraint.constraintsWithVisualFormat("V:|-10-[donelBtn(30)]-5-[splitLine(0.5)]-5-[pickView]-0-|", options: NSLayoutFormatOptions(rawValue: 0), metrics: nil, views: viewDict)
        cons += NSLayoutConstraint.constraintsWithVisualFormat("H:|-0-[pickView]-0-|", options: NSLayoutFormatOptions(rawValue: 0), metrics: nil, views: viewDict)
        cons += NSLayoutConstraint.constraintsWithVisualFormat("H:|-0-[splitLine]-0-|", options: NSLayoutFormatOptions(rawValue: 0), metrics: nil, views: viewDict)
        bgView.addConstraints(cons)
        // 添加监听
        cancelBtn.addTarget(self, action: #selector(cancel), forControlEvents: .TouchUpInside)
        donelBtn.addTarget(self, action: #selector(done), forControlEvents: .TouchUpInside)
        // 配色
        self.color = tintColor
        self.bgView.backgroundColor = bgColor
        backgroundColor = UIColor(white: 0, alpha: 0.5)
        self.hidden = true
        // 日历
        if Float(UIDevice.currentDevice().systemVersion) >= 8.0 {
            self.calendar = NSCalendar(calendarIdentifier: NSCalendarIdentifierChinese)
        } else {
            self.calendar = NSCalendar(identifier: NSChineseCalendar)
        }

    }
    // MARK: - 控制方法
    /**
     弹出view
     */
    func showInView() {
        UIView.animateWithDuration(0.5) {
            self.bgView.frame.origin.y = CGRectGetHeight(UIScreen.mainScreen().bounds) - self.bgView.frame.size.height
            self.hidden = false
        }
    }
    /**
     隐藏view
     */
    func hideFromView() {
        UIView.animateWithDuration(0.5, animations: {
            self.bgView.frame.origin.y = CGRectGetHeight(UIScreen.mainScreen().bounds)
        }) { (bool) in
            if bool {
                self.hidden = true
            }
        }
    }
    /**
     根据frame设置圆角
     */
    private func setupCornor(frame:CGRect) {
        let rect = CGRect(x: 0, y: 0, width: frame.size.width, height: frame.size.height)
        let maskPath = UIBezierPath(roundedRect: rect, byRoundingCorners: [.TopRight,.TopLeft], cornerRadii: CGSize(width: 15, height: 15))
        let maskLayer = CAShapeLayer()
        maskLayer.frame = rect
        maskLayer.path = maskPath.CGPath
        self.bgView.layer.mask = maskLayer
    }
    override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
        print(touches)
    }
    // MARK: - pickView方法
    
    /**
     初始化返回日期
     */
    private func initDate() {
        var startDay = 0
        var startHour = 0
        var startMinute = 0
        
        // 获取天数行数
        if self.minDate != nil && self.maxDate != nil && self.showOnlyValidDates {// 如果设置了最大最小日期
            // 创建一个包含天数,从最小日期到最大日期的组件
            let components = self.calendar!.components(.Day, fromDate: self.minDate!, toDate: self.maxDate!, options: NSCalendarOptions(rawValue: 0))
            // 拿到天数的行数
            nDays = components.day + 1
            
        } else {// 如果没有设置
            // 行数为最大
            nDays = Int(INT16_MAX)
        }
        
        // 最大的日期
        var dateToPresent:NSDate?
        
        if self.minDate?.compare(NSDate()) == NSComparisonResult.OrderedDescending {
            dateToPresent = self.minDate
        } else if self.maxDate?.compare(NSDate()) == NSComparisonResult.OrderedAscending {
            dateToPresent = self.maxDate
        } else {
            dateToPresent = NSDate()
        }
        // 创建一个包含天时分,从最早日期到最大日期的组件
        let todaysComponents = self.calendar!.components([.Day,.Hour,.Minute], fromDate: self.earliestPresentedDate, toDate: dateToPresent!, options: NSCalendarOptions(rawValue: 0))
        // 转换为时间戳并赋值
        startDay = todaysComponents.day * 60 * 60 * 24
        startHour = todaysComponents.hour * 60 * 60
        startMinute = todaysComponents.minute * 60
        // 计算总时间戳
        let timeInterval:NSTimeInterval = Double(startDay + startHour + startMinute)
        // 赋值给返回的日期
        self.date = NSDate(timeInterval: timeInterval, sinceDate: self.earliestPresentedDate)
    }
    /**
     根据日期滑动到对于的row
     
     - parameter date: 日期
     */
    private func showDateOnPicker(date:NSDate) {
        
        self.date = date
        // 创一个由年月日,最早的日期组成的组件
        var components = self.calendar!.components([NSCalendarUnit.Year,.Month,.Day], fromDate: self.earliestPresentedDate)
        // 根据组件从日历中拿到NSDate
        let fromDate = self.calendar!.dateFromComponents(components)
        // 创建一个日时分,从fromDate到需要显示的date的组件
        components = self.calendar!.components([.Day,.Hour,.Minute], fromDate: fromDate!, toDate: date, options: NSCalendarOptions(rawValue: 0))
        // 计算行数,在计算分钟和小时的时候,为避免滑动到第0行,加上x * (Int(INT16_MAX) / 120),其中x等于对于的进制
        let hoursRow = components.hour + 24 * (Int(INT16_MAX) / 120)
        let minutesRow = components.minute + 60 * (Int(INT16_MAX) / 120)
        let daysRow = components.day
        // 滑动到对于的行
        pickView.selectRow(daysRow, inComponent: 0, animated: true)
        pickView.selectRow(hoursRow, inComponent: 1, animated: true)
        pickView.selectRow(minutesRow, inComponent: 2, animated: true)
    }
    // MARK: - 按钮监听方法
    /**
     取消按钮
     */
    func cancel() {
        // 隐藏
        hideFromView()
    }
    /**
     确定按钮
     */
    func done() {
        // 获取系统当前时区
        let zone = NSTimeZone.systemTimeZone()
        // 计算与GMT时区的差
        let interval = zone.secondsFromGMTForDate(date!)
        // 加上差的时时间戳
        let localeDate = date?.dateByAddingTimeInterval(Double(interval))
        // 回调闭包
        didFinishSelect?(date: localeDate!)
        // 隐藏
        self.hideFromView()
    }
    // MARK: - 懒加载
    /// 取消按钮
    private lazy var cancelBtn:UIButton = {
        let btn = UIButton()
        btn.setTitle("取消", forState: .Normal)
        btn.setTitleColor(self.color, forState: .Normal)
        return btn
    }()
    /// 中间标题
    private lazy var titleLabel:UILabel = {
        let label = UILabel()
        label.text = "选择时间"
        label.textAlignment = NSTextAlignment.Center
        label.textColor = self.color
        return label
    }()
    /// 确定按钮
    private lazy var donelBtn:UIButton = {
        let btn = UIButton()
        btn.setTitle("确定", forState: .Normal)
        btn.setTitleColor(self.color, forState: .Normal)
        return btn
    }()
    /// 分割线
    private lazy var splitLine:UIView = {
        let sl = UIView()
        sl.backgroundColor = self.color
        return sl
    }()
    /// 时间选择器
    private lazy var pickView:UIPickerView = {
        let pv = UIPickerView()
        pv.delegate = self
        pv.dataSource = self
        return pv
    }()
    // 背景
    private lazy var bgView:UIView = UIView(frame: CGRect(x: 0, y: CGRectGetMaxY(self.frame), width: CGRectGetWidth(self.frame), height: 250))
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
}
// MARK: - UIPickerViewDelegate+UIPickerViewDataSource
extension AbbDatePickView:UIPickerViewDelegate,UIPickerViewDataSource{
    func numberOfComponentsInPickerView(pickerView: UIPickerView) -> Int {
        return 3
    }
    
    func pickerView(pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if component == 0 {
            return nDays ?? 0
        }
        else if component == 1 {
            return Int(INT16_MAX)
        } else {
            return Int(INT16_MAX)
        }
    }
    
    func pickerView(pickerView: UIPickerView, widthForComponent component: Int) -> CGFloat {
        switch component {
        case 0:
            return 120
        case 1:
            return 60
        case 2:
            return 60
        default:
            return 0
        }
    }
    func pickerView(pickerView: UIPickerView, rowHeightForComponent component: Int) -> CGFloat {
        return 35
    }
    
    func pickerView(pickerView: UIPickerView, viewForRow row: Int, forComponent component: Int, reusingView view: UIView?) -> UIView {
        // 定义一个label用于展示时间
        let dateLabel = UILabel()
        // 字体大小
        dateLabel.font = UIFont.systemFontOfSize(25)
        // 字体颜色
        dateLabel.textColor = UIColor.whiteColor()
        // label背景色
        dateLabel.backgroundColor = UIColor.clearColor()
        
        // 隐藏黑线
        pickerView.subviews[1].hidden = true
        pickerView.subviews[2].hidden = true
        
        if component == 0 {// 天数
            // 根据当前的行数转换为时间戳,记录当前行数所表示的日期
            let aDate = NSDate(timeInterval: Double(row * 24 * 60 * 60), sinceDate: self.earliestPresentedDate)
            // 创建一个有纪元年月日组成的,当前时间的组件
            var components = self.calendar!.components([.Era,.Year,.Month,.Day], fromDate: NSDate())
            // 根据组件从日历里拿到今天的NSDate()
            let toDay = self.calendar!.dateFromComponents(components)
            // 组件变为由当前行数所表示的日期组成的组件
            components = self.calendar!.components([.Era,.Year,.Month,.Day], fromDate: aDate)
            // 根据组件从日历拿到当前行数表示的NSDate
            let otherDate = self.calendar!.dateFromComponents(components)
            // 如果今天的NSDate等于当前行数表示的NSDate,就设置文字为今天
            if toDay!.isEqualToDate(otherDate!) {
                dateLabel.text = "今天"
            } else {
                // 如果不是,创建一个NSDateFormatter
                let formatter = NSDateFormatter()
                // 地区设置
                formatter.locale = NSLocale.currentLocale()
                // 日期格式设置
                formatter.dateFormat = "M月d日"
                // label文字设置
                dateLabel.text = formatter.stringFromDate(aDate)
            }
            dateLabel.textAlignment = NSTextAlignment.Center
        } else if component == 1 {// 小时
            // 小时的范围0-23,长度是24
            let max = self.calendar!.maximumRangeOfUnit(NSCalendarUnit.Hour).length
            // label文字
            dateLabel.text = String(format:"%02ld",row % max)
            dateLabel.textAlignment = NSTextAlignment.Left
        } else if component == 2 {// 分钟
            // 分钟的范围0-59,长度是60
            let max = self.calendar!.maximumRangeOfUnit(NSCalendarUnit.Minute).length
            dateLabel.text = String(format:"%02ld",row % max)
            dateLabel.textAlignment = NSTextAlignment.Left
        }
        return dateLabel
    }
    
    func pickerView(pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        // 选择的日的行数
        let daysRow = pickerView.selectedRowInComponent(0)
        // 根据行数转换为时间戳
        let chosenDate = NSDate(timeInterval: Double(daysRow * 24 * 60 * 60), sinceDate: self.earliestPresentedDate)
        // 选择的小时的行数
        let hoursRow = pickerView.selectedRowInComponent(1)
        // 选择的分钟的行数
        let minutesRow = pickerView.selectedRowInComponent(2)
        // 根据选择的日期的时间戳,创建一个有年月日的日历组件
        let components = self.calendar!.components([.Day,.Month,.Year], fromDate: chosenDate)
        // 设置组件的小时
        components.hour = hoursRow % 24
        // 设置组件的分钟
        components.minute = minutesRow % 60
        // 根据组件从日历中拿到对应的NSDate,赋值给date
        self.date = self.calendar!.dateFromComponents(components)
        // 比较date与限定的最大最小时间,如果超过最大时间或小于最小时间,就回滚到有效时间内
        if self.date!.compare(self.minDate!) == NSComparisonResult.OrderedAscending {
            self.showDateOnPicker(self.minDate!)
        } else if self.date!.compare(self.maxDate!) == NSComparisonResult.OrderedDescending {
            self.showDateOnPicker(self.maxDate!)
        }
    }
    
}

